#!/bin/bash

sleep 30
mkdir -p /mnt/SD

if [ -e /dev/ubi0_0 ]; then
    mount /dev/mmcblk0p1 /mnt/SD;
else
    mount /dev/mmcblk1p1 /mnt/SD;
fi

if mountpoint -q /mnt/SD; then
    mkdir -p /mnt/SD/Datalog/syslog/
else
    exit 1
fi

Date()
{
	date=`date +%Y-%m-%d-%H-%M-%S`;
	echo "$date" >> /mnt/SD/Datalog/syslog/$1;
}

monitorResources()
{
	echo -n "1. " >> /mnt/SD/Datalog/syslog/$1;
	Date $1;
	CPU=`top -b -d1 -n1 | grep Cpu | awk '{print $2}' | cut -d'%' -f1`;
	MEM=`free | grep Mem | awk '{print $3/$2 * 100.0}'`;
	echo "   CPU Usage(%): $CPU" >> /mnt/SD/Datalog/syslog/$1;
	echo "   Memory Usage(%): $MEM" >> /mnt/SD/Datalog/syslog/$1;
	echo "=============================================" >> /mnt/SD/Datalog/syslog/$1;
}

testMQTTConnection()
{
    if [ ! -s /app/runtime/project/plugin/server/MQTTService.xml ]; then
        echo "MQTT Client is not set up." >> /mnt/SD/Datalog/syslog/$1
    else
        declare -A testedBrokers
        while IFS= read -r line
        do
            if [[ $line == *"<Broker>"* ]]; then
                broker=""
            elif [[ $line == *"</Broker>"* ]]; then
                if [[ ! ${testedBrokers[$broker]} ]]; then
                    testedBrokers[$broker]=1
                    ip=$(echo "$broker" | awk -F'[<>]' '/<IP>/{print $3}')
                    port=$(echo "$broker" | awk -F'[<>]' '{print $7}')
                    if [[ -z "$ip" || -z "$port" ]]; then
                        continue
                    fi
                    echo -n "2. " >> /mnt/SD/Datalog/syslog/$1
                    Date $1
                    echo "   Testing MQTT broker connection at IP: $ip and Port: $port" >> /mnt/SD/Datalog/syslog/$1
                    /app/MQTT/Broker/mqtt_connect -h $ip -p $port -t "/test" -m "Test message" >> /mnt/SD/Datalog/syslog/$1 2>&1
                    if [ $? -eq 0 ]; then
                        echo "   MQTT broker connection test passed." >> /mnt/SD/Datalog/syslog/$1
                    else
                        echo "   MQTT broker connection test failed." >> /mnt/SD/Datalog/syslog/$1
                    fi
                    echo "=============================================" >> /mnt/SD/Datalog/syslog/$1
                fi
            else
                broker+="$line"
            fi
        done < /app/runtime/project/plugin/server/MQTTService.xml
    fi
}

testWebServer()
{
    echo -n "3. " >> /mnt/SD/Datalog/syslog/$1
    Date $1
    WebServerIP="127.0.0.1"
    WebServerPort="80"
    echo "   Testing web server connection at IP: $WebServerIP" >> /mnt/SD/Datalog/syslog/$1
    /app/System/curl -I http://$WebServerIP:$WebServerPort >> /mnt/SD/Datalog/syslog/$1 2>&1
    echo "=============================================" >> /mnt/SD/Datalog/syslog/$1
}

monitorTCP()
{
    echo -n "4. " >> /mnt/SD/Datalog/syslog/$1
    Date $1
    TCPStatus=`netstat -an | grep tcp`
    if [ "$TCPStatus" != "" ]; then
        echo "   TCP Status: " >> /mnt/SD/Datalog/syslog/$1
        echo "$TCPStatus" >> /mnt/SD/Datalog/syslog/$1
    else
        echo "   No TCP connections." >> /mnt/SD/Datalog/syslog/$1
    fi
    echo "=============================================" >> /mnt/SD/Datalog/syslog/$1
}

monitorNetwork()
{ 
    echo -n "5. " >> /mnt/SD/Datalog/syslog/$1
    Date $1
    echo "   Checking network interface: $iface" >> /mnt/SD/Datalog/syslog/$1

    for iface in eth0 eth1; do
        if ! ip link show $iface > /dev/null 2>&1; then
            continue
        fi
        IP=$(ip -4 addr show $iface | grep -oP '(?<=inet\s)\d+(\.\d+){3}')
        echo "   IP Address: $IP" >> /mnt/SD/Datalog/syslog/$1
        GATEWAY=$(ip route | grep default | grep $iface | awk '{print $3}')
        if [ -n "$GATEWAY" ]; then
            echo "   Gateway: $GATEWAY" >> /mnt/SD/Datalog/syslog/$1
            if ping -c 1 $GATEWAY &> /dev/null
            then
                echo "   Ping to gateway: Success" >> /mnt/SD/Datalog/syslog/$1
            else
                echo "   Ping to gateway: Failed" >> /mnt/SD/Datalog/syslog/$1
            fi
        else
            echo "   No gateway set for this interface." >> /mnt/SD/Datalog/syslog/$1
        fi
        echo "=============================================" >> /mnt/SD/Datalog/syslog/$1
    done
}

Date_File=`date +%Y-%m-%d`;

interval=${1:-30}  

while true
do
	New_Date_File=`date +%Y-%m-%d`;
	if [ "$New_Date_File" != "$Date_File" ]; then
		Date_File=$New_Date_File;
	fi

    if [ ! -f /mnt/SD/Datalog/syslog/$Date_File ]; then
		touch /mnt/SD/Datalog/syslog/$Date_File
	fi

	echo "============== Data logger ==================" >> /mnt/SD/Datalog/syslog/$Date_File;
    monitorResources $Date_File;
    testMQTTConnection $Date_File
    testWebServer $Date_File
    monitorTCP $Date_File
    monitorNetwork $Date_File
    
	logfiles=(/mnt/SD/Datalog/syslog/*)
	while [ ${#logfiles[@]} -gt 14 ]
	do
		rm "${logfiles[0]}"
		logfiles=(/mnt/SD/Datalog/syslog/*)
	done

	sleep $(( (interval*60 - 10#$(date +%M)*60 - 10#$(date +%S) + interval*60 ) % (interval*60) ))
    
done
